from .thumbnail_generator import ThumbnailGenerator
from .text_optimizer import TextOptimizer
from .background_manager import BackgroundManager
from .font_manager import FontManager

__version__ = "1.1.0"
__all__ = [
    "ThumbnailGenerator",
    "TextOptimizer",
    "BackgroundManager",
    "FontManager",
]